@@ -29,7 +29,6 @@
 using System;
 using System.Data;
 using System.IO;
-using System.Text.RegularExpressions;
 using System.Collections.Generic;
 using System.Threading;
 using Mono.Unix;
@@ -48,31 +47,15 @@ using Banshee.Metadata;
 using Banshee.MediaEngine;
 using Banshee.Collection;
 using Banshee.ServiceStack;
-using Banshee.PlaybackController;
  
 namespace Banshee.Lastfm.Radio
 {
-    public class StationSource : Source, ITrackModelSource, IUnmapableSource, IDisposable, IBasicPlaybackController
+    public class StationSource : Source, ITrackModelSource, IUnmapableSource, IDisposable
     {
         private static string generic_name = Catalog.GetString ("Last.fm Station");
-
-        public static StationSource CreateFromUrl (LastfmSource lastfm, string url)
-        {
-            foreach (StationType type in StationType.Types) {
-                string regex = Regex.Escape (type.GetStationFor ("XXX")).Replace ("XXX", "([^\\/]+)");
-                Match match = Regex.Match (url, regex);
-                if (match.Groups.Count == 2 && match.Groups[0].Captures.Count > 0) {
-                    Log.DebugFormat ("Creating last.fm station from url {0}", url);
-                    string arg = match.Groups[1].Captures[0].Value;
-                    StationSource station = new StationSource (lastfm, arg, type.Name, arg);
-                    lastfm.AddChildSource (station);
-                    station.NotifyUser ();
-                }
-            }
-            return null;
-        }
         
-        private MemoryTrackListModel track_model;
+        private LastfmTrackListModel track_model;
+        private SourceMessage status_message;
         
         private LastfmSource lastfm;
         public LastfmSource LastfmSource {
@@ -110,14 +93,6 @@ namespace Banshee.Lastfm.Radio
 
         private int dbid;
         
-        protected override string TypeUniqueId {
-            get { return Convert.ToString (dbid); }
-        }
-        
-        public System.Collections.Generic.IEnumerable<Banshee.Collection.Database.IFilterListModel> FilterModels {
-            get { yield break; }
-        }
-        
         // For StationSources that already exist in the db
         protected StationSource (LastfmSource lastfm, int dbId, string name, string type, string arg, int playCount) : base (generic_name, name, 150)
         {
@@ -128,7 +103,7 @@ namespace Banshee.Lastfm.Radio
             PlayCount = playCount;
             Station = Type.GetStationFor (arg);
 
-            StationInitialize ();
+            Initialize ();
         }
 
         public StationSource (LastfmSource lastfm, string name, string type, string arg) : base (generic_name, name, 150)
@@ -140,14 +115,14 @@ namespace Banshee.Lastfm.Radio
 
             Save ();
 
-            StationInitialize ();
+            Initialize ();
         }
 
-        private void StationInitialize ()
+        private void Initialize ()
         {
-            track_model = new MemoryTrackListModel ();
+            track_model = new LastfmTrackListModel ();
 
-            ServiceManager.PlayerEngine.ConnectEvent (OnPlayerEvent, PlayerEvent.StateChange);
+            ServiceManager.PlayerEngine.StateChanged += OnPlayerStateChanged;
             lastfm.Connection.StateChanged += HandleConnectionStateChanged;
 
             Properties.SetString ("GtkActionPath", "/LastfmStationSourcePopup");
@@ -159,7 +134,7 @@ namespace Banshee.Lastfm.Radio
 
         public void Dispose ()
         {
-            ServiceManager.PlayerEngine.DisconnectEvent (OnPlayerEvent);
+            ServiceManager.PlayerEngine.StateChanged -= OnPlayerStateChanged;
             lastfm.Connection.StateChanged -= HandleConnectionStateChanged;
         }
 
@@ -203,6 +178,9 @@ namespace Banshee.Lastfm.Radio
         public override void Activate ()
         {
             base.Activate ();
+            //shuffle = (action_service.GlobalActions ["ShuffleAction"] as ToggleAction).Active;
+            //(action_service.GlobalActions ["ShuffleAction"] as ToggleAction).Active = false;
+            //Globals.ActionManager["ShuffleAction"].Sensitive = false;
 
             //action_service.GlobalActions ["PreviousAction"].Sensitive = false;
 
@@ -228,6 +206,9 @@ namespace Banshee.Lastfm.Radio
 
         public override void Deactivate ()
         {
+            //(Globals.ActionManager["ShuffleAction"] as ToggleAction).Active = shuffle;
+            //Globals.ActionManager["ShuffleAction"].Sensitive = true;
+
             //Globals.ActionManager["PreviousAction"].Sensitive = true;
         }
 
@@ -254,16 +235,43 @@ namespace Banshee.Lastfm.Radio
             }
         }
 
-        public override void SetStatus (string message, bool error)
+        private void SetStatus (string message, bool error)
         {
-            base.SetStatus (message, error);
-            LastfmSource.SetStatus (status_message, lastfm, error, ConnectionState.Connected);
+            SetStatus (message, error, ConnectionState.Connected);
         }
 
-        public void SetStatus (string message, bool error, ConnectionState state)
+        private void SetStatus (string message, bool error, ConnectionState state)
         {
-            base.SetStatus (message, error);
-            LastfmSource.SetStatus (status_message, lastfm, error, state);
+            if (status_message == null) {
+                status_message = new SourceMessage (this);
+                PushMessage (status_message);
+            }
+            
+            string status_name = String.Format ("<i>{0}</i>", GLib.Markup.EscapeText (Name));
+            
+            status_message.FreezeNotify ();
+            status_message.Text = String.Format (GLib.Markup.EscapeText (message), status_name);
+            status_message.CanClose = !error;
+            status_message.IsSpinning = !error;
+            status_message.SetIconName (error ? "dialog-error" : null);
+            status_message.ClearActions ();
+            
+            if (error && (state == ConnectionState.NoAccount || state == ConnectionState.InvalidAccount)) {
+                status_message.AddAction (new MessageAction (Catalog.GetString ("Account Settings"),
+                    delegate { lastfm.Actions.ShowLoginDialog (); }));
+                status_message.AddAction (new MessageAction (Catalog.GetString ("Join Last.fm"),
+                    delegate { lastfm.Account.SignUp (); }));
+            }
+            
+            status_message.ThawNotify ();
+        }
+
+        private void HideStatus ()
+        {
+            if (status_message != null) {
+                RemoveMessage (status_message);
+                status_message = null;
+            }
         }
 
         /*public override void ShowPropertiesDialog ()
@@ -292,30 +300,6 @@ namespace Banshee.Lastfm.Radio
                     current_track = i;
             }
         }
-        
-#region IBasicPlaybackController
-
-        void IBasicPlaybackController.First ()
-        {
-            ((IBasicPlaybackController)this).Next (false);
-        }
-        
-        private bool playback_requested;    
-        void IBasicPlaybackController.Next (bool restart)
-        {
-            TrackInfo next = NextTrack;
-            if (next != null) {
-                ServiceManager.PlayerEngine.OpenPlay (next);
-            }  else {
-                playback_requested = true;
-            }
-        }
-        
-        void IBasicPlaybackController.Previous (bool restart)
-        {
-        }
-        
-#endregion
 
         public TrackInfo NextTrack {
             get { return GetTrack (current_track + 1); }
@@ -371,12 +355,10 @@ namespace Banshee.Lastfm.Radio
                             track_model.Reload ();
                             OnUpdated ();
 
-                            if (playback_requested) {
-                                if (this == ServiceManager.PlaybackController.Source ) {
-                                    ((IBasicPlaybackController)this).Next (false);
-                                }
+                            /*if (playback_requested) {
+                                StartPlayback ();
                                 playback_requested = false;
-                            }
+                            }*/
                         });
                     }
                 } else {
@@ -387,10 +369,9 @@ namespace Banshee.Lastfm.Radio
             });
         }
 
-        private void OnPlayerEvent (PlayerEventArgs args)
+        private void OnPlayerStateChanged (object o, PlayerEngineStateArgs args)
         {
-            if (((PlayerEventStateChangeArgs)args).Current == PlayerState.Loaded && 
-                track_model.Contains (ServiceManager.PlayerEngine.CurrentTrack)) {
+            if (args.State == PlayerEngineState.Loaded && track_model.Contains (ServiceManager.PlayerEngine.CurrentTrack)) {
                 CurrentTrack = ServiceManager.PlayerEngine.CurrentTrack;
 
                 lock (track_model) {
@@ -445,11 +426,6 @@ namespace Banshee.Lastfm.Radio
             );
         }
 
-        public override bool CanRename {
-            get { return true; }
-        }
-
-
 #region ITrackModelSource Implementation
 
         public TrackListModel TrackModel {
@@ -478,10 +454,6 @@ namespace Banshee.Lastfm.Radio
             throw new Exception ("Should not call DeleteSelectedTracks on StationSource");
         }
 
-        public bool CanAddTracks {
-            get { return false; }
-        }
-
         public bool CanRemoveTracks {
             get { return false; }
         }
@@ -530,6 +502,14 @@ namespace Banshee.Lastfm.Radio
             Save ();
         }
         
+        public override bool CanSearch {
+            get { return false; }
+        }
+
+        public override int Count {
+            get { return 0; }
+        }
+
         public override bool HasProperties {
             get { return true; }
         }
